<?php

// License GNU GPL
// Author: Sbastien Beyou
// Include some parts of the LDAP extension for MediaWiki, which have been adapted.

//Server address and type of encryption
# $ggLDAPServerNames = "example.com";
# $ggLDAPEncryptionType = "tls";
# $ggLDAPOptions = array();
# $ggLDAPDebug = false;

//User and password used for proxyagent access.
//Please use a user with limited access, NOT your directory manager!
# $ggLDAPProxyAgent = "cn=proxyagent,ou=profile,dc=LDAP,dc=example,dc=com";
# $ggLDAPProxyAgentPassword = "S0M3L0ngP@$$w0r6ofS0meV@rie222y!";

//Search filter.
# $ggLDAPSearchAttributes = "uid";
 
//Base DNs. User base DNs will be used if available; if it is not defined, the search
//will default to $wgLDAPBaseDNs
# $ggLDAPBaseDNs = "dc=LDAP,dc=example,dc=com";
# $ggLDAPUserBaseDNs = "ou=people,dc=LDAP,dc=example,dc=com";

//User and password of the privileged user who write in the directory
# $ggLDAPWriterDN = "uid=priviledgedUser,ou=people,dc=LDAP,dc=example,dc=com";
# $ggLDAPWriterPassword = "S0M3L0ngP@$$w0r6ofS0meV@rie222y!";

//A location to add users to if you are using $ggLDAPSearchAttributes and $ggLDAPAddLDAPUsers.
# $ggLDAPWriteLocation = "ou=people,dc=LDAP,dc=example,dc=com";
 
//Options for adding users, and/or updating user preferences in LDAP.
# $ggLDAPAddLDAPUsers = true;
# $ggLDAPUpdateLDAP = true;

//Liste des groupes  mettre  jour
//array( "identifiant du champ dynamique", "valeur pour tre inscrit", "DN du groupe" ),
# $ggLDAPManagedServices = array(
#   array( "13-1", "0", "ou=mailman,ou=groups,dc=example,dc=com"   ),
#   array( "14-1", "0", "ou=mediawiki,ou=groups,dc=example,dc=com" ),
# );

//constants for search base
define("USERDN", 1);
define("DEFAULTDN", 2);

class LDAPGalette extends AuthGalette {
	
	/**
	 * Add a user in the external database
	 * Ajouter un utilisateur dans la base de donnes externe
	 * 
	 * @return bool
	 * @public
	 */
	function addExternalUser() {
		
		if( !$this->getAdmin() ) return false;
		
		global $ggLDAPWriterDN, $ggLDAPWriterPassword, $ggLDAPAddLDAPUsers, $ggLDAPDebug;
		global $ggLDAPSearchAttributes, $ggLDAPWriteLocation;
		
		if( $ggLDAPDebug ) print "Entering addExternalUser<br />\n";
		if ( !isset( $ggLDAPAddLDAPUsers ) || !$ggLDAPAddLDAPUsers ) return false;
		if ( !isset( $ggLDAPWriterDN ) ) return false;
		
		$ldapconn = $this->connect();
		if( !$ldapconn ) {
			if( $ggLDAPDebug ) print "Failed to connect to LDAP<br />\n";
			return false;
		}
		
		$pseudo = iconv("ISO-8859-1", "UTF-8", $this->getField("pseudo_adh"));
		$userdn = $this->getSearchString( $ldapconn, $pseudo );
		if ( "" == $userdn ) {
			if( $ggLDAPDebug ) print "userdn is blank, attempting to use wgLDAPWriteLocation<br />\n";
			if ( isset( $ggLDAPWriteLocation ) ) {
				if( $ggLDAPDebug ) print "ggLDAPWriteLocation is set, using that<br />\n";
				$userdn = $ggLDAPSearchAttributes . "=" . $pseudo . "," . $ggLDAPWriteLocation;
			} else {
				if( $ggLDAPDebug ) print "wgLDAPWriteLocation is not set, failing<br />\n";
				//getSearchString will bind, but will not unbind
				@ldap_unbind();
				return false;
			}
		}
		
		if( $ggLDAPDebug ) print "Binding as the writerDN<br />\n";
		
		$bind = $this->bindAs( $ldapconn, $ggLDAPWriterDN, $ggLDAPWriterPassword );
		if ( !$bind ) {
			if( $ggLDAPDebug ) print "Failed to bind as the writerDN; add failed<br />\n";
			return false;
		}
		
		//Set up LDAP attributes
		$values = array();
		$values["uid"] = $pseudo;
		
		//sn is required for objectclass inetorgperson
		$values["sn"] = $pseudo;
		
		if ( "" != $this->getField("email_adh") ) $values["mail"] = iconv("ISO-8859-1", "UTF-8", $this->getField("email_adh"));
		
		if ( "" != $this->getField("prenom_adh") || "" != $this->getField("nom_adh") ) {
			$values["cn"] = "";
			if( $this->getField("prenom_adh") != "" ) $values["cn"] .= $this->getField("prenom_adh");
			if( $this->getField("prenom_adh") != "" && $this->getField("nom_adh") != "" ) $values["cn"] .= " ";
			if( $this->getField("nom_adh") != "" ) $values["cn"] .= $this->getField("nom_adh");
			$values["cn"] = iconv("ISO-8859-1", "UTF-8", $values["cn"]);
		}
		else $values["cn"] = $pseudo;
		
		if ( "" != $this->getField("pref_lang") ) {
			if( $this->getField("pref_lang") == "french" ) $values["preferredlanguage"] = "fr";
			if( $this->getField("pref_lang") == "english" ) $values["preferredlanguage"] = "en";
			if( $this->getField("pref_lang") == "spanish" ) $values["preferredlanguage"] = "es";
		}
		
		$values["userpassword"] = $this->getPasswordHash( iconv("ISO-8859-1", "UTF-8", $this->getField("password")) );
		
		$values["objectclass"] = "inetorgperson";
		
		if( $ggLDAPDebug ) print "Adding user $userdn<br />\n";
		if ( @ldap_add( $ldapconn, $userdn, $values ) ) {
			if( $ggLDAPDebug ) print "Successfully added user<br />\n";
			@ldap_unbind();
			return true;
		} else {
			if( $ggLDAPDebug ) print "Failed to add user<br />\n";
			@ldap_unbind();
			return false;
		}
		
		return true;
	}
	
	/**
	 * Update the external database
	 * Mettre  jour la base de donnes externe
	 * 
	 * @return bool
	 * @public
	 */
	function updateExternalUser() {
		global $ggLDAPWriterDN, $ggLDAPWriterPassword, $ggLDAPUpdateLDAP, $ggLDAPGroupBaseDNs, $ggLDAPManagedServices, $ggLDAPDebug;
		
		if( $ggLDAPDebug ) print "Entering updateExternalDB<br />\n";
		if ( !isset( $ggLDAPUpdateLDAP ) || !$ggLDAPUpdateLDAP ) return false;
		if ( !isset( $ggLDAPWriterDN ) ) return false;
		
		$ldapconn = $this->connect();
		if( !$ldapconn ) {
			if( $ggLDAPDebug ) print "Failed to connect to LDAP<br />\n";
			return false;
		}
		
		$userdn = $this->getSearchString( $ldapconn, iconv("ISO-8859-1", "UTF-8", $this->getField("pseudo_adh")) );
		if ( !$userdn ) {
			return false;
		}
		
		if( $ggLDAPDebug ) print "Binding as the writerDN<br />\n";
		$bind = $this->bindAs( $ldapconn, $ggLDAPWriterDN, $ggLDAPWriterPassword );
		if ( !$bind ) {
			return false;
		}
		
		
		// Mise  jour de l'utilisateur dans LDAP
		$values = array();
		if ( "" != $this->getField("email_adh") ) $values["mail"] = iconv("ISO-8859-1", "UTF-8", $this->getField("email_adh"));
		
		if ( "" != $this->getField("pseudo_adh") && $this->getAdmin() ) $values["displayname"] = iconv("ISO-8859-1", "UTF-8", $this->getField("pseudo_adh"));
		
		if ( "" != $this->getField("prenom_adh") || "" != $this->getField("nom_adh") ) {
			$values["cn"] = "";
			if( $this->getField("prenom_adh") != "" ) $values["cn"] .= $this->getField("prenom_adh");
			if( $this->getField("prenom_adh") != "" && $this->getField("nom_adh") != "" ) $values["cn"] .= " ";
			if( $this->getField("nom_adh") != "" ) $values["cn"] .= $this->getField("nom_adh");
			$values["cn"] = iconv("ISO-8859-1", "UTF-8", $values["cn"]);
		}
		
		if ( "" != $this->getField("pref_lang") ) {
			if( $this->getField("pref_lang") == "french" ) $values["preferredlanguage"] = "fr";
			if( $this->getField("pref_lang") == "english" ) $values["preferredlanguage"] = "en";
			if( $this->getField("pref_lang") == "spanish" ) $values["preferredlanguage"] = "es";
		}
		
		if( isset($this->fields["password"]) && $this->getField("password") != "" ) {
			$values["userpassword"] = $this->getPasswordHash( iconv("ISO-8859-1", "UTF-8", $this->getField("password")) );
		}
		
		if ( 0 != sizeof( $values ) && ldap_modify( $ldapconn, $userdn, $values ) ) {
			if( $ggLDAPDebug ) print "Successfully modified the user's attributes<br />\n";
		} else {
			if( $ggLDAPDebug ) print "Failed to modify the user's attributes<br />\n";
			@ldap_unbind();
			return false;
		}
		
		// Abonnements
		$ret = $this->updateGroups( $ldapconn, $userdn );
		
		@ldap_unbind();
		return $ret;
	}
	
	/**
	 * Connect to LDAP
	 * 
	 * @return resource
	 * @access private
	 */
	private function connect() {
		global $ggLDAPServerNames;
		global $ggLDAPEncryptionType;
		global $ggLDAPOptions;
		global $ggLDAPDebug;
		
		if( $ggLDAPDebug ) print "Entering Connect<br />\n";
		
		if ( !extension_loaded( "ldap" ) ) {
			print "It looks like you are issing LDAP support; please ensure you have either compiled LDAP support in, or have enabled the module. If the authentication is working for you, the plugin isn't properly detecting the LDAP module, and you can safely ignore this message.";
		}
		
		//If the admin didn't set an encryption type, we default to tls
		if ( isset( $ggLDAPEncryptionType ) ) {
			$encryptionType = $ggLDAPEncryptionType;
		} else {
			$encryptionType = "tls";
		}
		
		//Set the server string depending on whether we use ssl or not
		switch( $encryptionType ) {
		case "ssl":
			if( $ggLDAPDebug ) print "Using SSL<br />\n";
			$serverpre = "ldaps://";
			break;
		default:
			if( $ggLDAPDebug ) print "Using TLS or not using encryption.<br />\n";
			$serverpre = "ldap://";
		}
		
		//Make a space seperated list of server strings with the ldap:// or ldaps://
		//string added.
		$servers = "";
		$tmpservers = $ggLDAPServerNames;
		$tok = strtok( $tmpservers, " " );
		while ( $tok ) {
			$servers = $servers . " " . $serverpre . $tok;
			$tok = strtok( " " );
		}
		$servers = rtrim($servers);
		
		if( $ggLDAPDebug ) print "Using servers: $servers<br />\n";
		
		//Connect and set options
		$ldapconn = @ldap_connect( $servers );
		ldap_set_option( $ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
		ldap_set_option( $ldapconn, LDAP_OPT_REFERRALS, 0);
		
		if ( isset( $ggLDAPOptions ) ) {
			$options = $ggLDAPOptions;
			foreach ( $options as $key => $value ) {
				if ( !ldap_set_option( $ldapconn, constant( $key ), $value ) && $ggLDAPDebug ) {
					print "Can't set option to LDAP! Option code and value: " . $key . "=" . $value . "<br />\n";
				}
			}
		}
		
		//TLS needs to be started after the connection is made
		if ( $encryptionType == "tls" ) {
			if( $ggLDAPDebug ) print "Using TLS<br />\n";
			if ( !ldap_start_tls( $ldapconn ) ) {
				if( $ggLDAPDebug ) print "Failed to start TLS.<br />\n";
				return;
			}
		}
		
		if( $ggLDAPDebug ) print "Connected successfully<br />\n";
		
		return $ldapconn;
	}
	
	/**
	 * Gets the searchstring for a user based upon settings for the domain.
	 * Returns a full DN for a user.
	 * 
	 * @param resource $ldapconn
	 * @param string $username
	 * @return string
	 * @access private
	 */
	private function getSearchString( $ldapconn, $username ) {
		global $ggLDAPProxyAgent, $ggLDAPProxyAgentPassword, $ggLDAPDebug;
		
		if( $ggLDAPDebug ) print "Entering getSearchString<br />\n";
		
		//This is a proxy bind, or an anonymous bind with a search
		if ( isset( $ggLDAPProxyAgent ) ) {
			//This is a proxy bind
			if( $ggLDAPDebug ) print "Doing a proxy bind<br />\n";
			$bind = $this->bindAs( $ldapconn, $ggLDAPProxyAgent, $ggLDAPProxyAgentPassword );
		} else {
			//This is an anonymous bind
			if( $ggLDAPDebug ) print "Doing an anonymous bind<br />\n";
			$bind = $this->bindAs( $ldapconn );
		}
		
		if ( !$bind ) {
			if( $ggLDAPDebug ) print "Failed to bind<br />\n";
			return "";
		}
		
		$userdn = $this->getUserDN( $ldapconn, $username );
		if( $ggLDAPDebug ) print "userdn is: $userdn<br />\n";
		return $userdn;
	}
	
	/**
	 * Gets the DN of a user based upon settings for the domain.
	 * You must bind to the server before calling this.
	 * 
	 * @param resource $ldapconn
	 * @param string $username
	 * @return string
	 * @access private
	 */
	private function getUserDN( $ldapconn, $username ) {
		global $ggLDAPSearchAttributes, $ggLDAPDebug;
		
		if( $ggLDAPDebug ) print "Entering getUserDN<br />\n";
		
		//we need to do a subbase search for the entry
		$filter = "(" . $ggLDAPSearchAttributes . "=" . $this->getLdapEscapedString( $username ) . ")";
		if( $ggLDAPDebug ) print "Created a regular filter: $filter<br />\n";
		
		$attributes = array( "*" );
		$base = $this->getBaseDN( USERDN );
		
		if( $ggLDAPDebug ) print "Using base: $base<br />\n";
		
		$entry = @ldap_search( $ldapconn, $base, $filter, $attributes );
		if ( !$entry ) {
			if( $ggLDAPDebug ) print "Couldn't find an entry<br />\n";
			return "";
		}
		
		$info = @ldap_get_entries( $ldapconn, $entry );
		
		$userdn = $info[0]["dn"];
		return $userdn;
	}
	
	/**
	 * Returns a basedn by the type of entry we are searching for.
	 * 
	 * @param int $type
	 * @return string
	 * @access private
	 */
	private function getBaseDN ( $type ) {
		global $ggLDAPBaseDNs, $ggLDAPUserBaseDNs, $ggLDAPDebug;
		
		if( $ggLDAPDebug ) print "Entering getBaseDN<br />\n";
		
		$ret = "";
		switch( $type ) {
			case USERDN:
				if ( isset( $ggLDAPUserBaseDNs ) ) {
					$ret = $ggLDAPUserBaseDNs;
				}
				break;
			case DEFAULTDN:
				if ( isset( $ggLDAPBaseDNs ) ) {
					$ret = $ggLDAPBaseDNs;
					if( $ggLDAPDebug ) print "basedn is $ret<br />\n";
					return $ret;
				} else {
					if( $ggLDAPDebug ) print "basedn is not set.<br />\n";
					return "";
				}
				break;
		}
		
		if ( $ret == "" ) {
			if( $ggLDAPDebug ) print "basedn is not set for this type of entry, trying to get the default basedn.<br />\n";
			// We will never reach here if $type is self::DEFAULTDN, so to avoid code
			// code duplication, we'll get the default by re-calling the function.
			return $this->getBaseDN( DEFAULTDN );
		} else {
			if( $ggLDAPDebug ) print "basedn is $ret<br />\n";
			return $ret;
		}
	}
	
	/**
	 * Update all possible subscriptions/groups
	 * 
	 * @param resource $ldapconn
	 * @param string $userdn
	 * @return bool
	 * @access private
	 */
	private function updateGroups( $ldapconn, $userdn ) {
		global $ggLDAPManagedServices, $ggLDAPDebug;
		
		if( !$this->getAdmin() ) return true;
		
		foreach( $ggLDAPManagedServices as $tableau ) {
			list( $champ, $valeur, $groupe ) = $tableau;
			
			// Lecture des valeurs existantes
			$entry = ldap_read( $ldapconn, $groupe, "objectClass=groupofuniquenames", array("uniquemember") );
			$info = ldap_get_entries( $ldapconn, $entry );
			if( $info["count"] != 1 ) {
				print "LDAP configuration error ".$info["count"];
				return false;
			}
			
			// Mise  jour de l'attribut uniqueMember
			$attr = array( "uniquemember" => array_diff_key( $info[0]["uniquemember"], array("count"=>0) ) );
			
			if( $this->getField("dyn".$champ) ===  $valeur ) {
				if( in_array( $userdn, $attr["uniquemember"] ) ) continue;
				else $attr["uniquemember"][] = $userdn;
			}
			else {
				if( !in_array( $userdn, $attr["uniquemember"] ) ) continue;
				else $attr["uniquemember"] = array_values( array_diff( $attr["uniquemember"], array($userdn) ) );
			}
			
			// Enregistrement
			if( !ldap_modify( $ldapconn, $groupe, $attr ) ) {
				if( $ggLDAPDebug ) print "Failed to modify the user's group $groupe<br />\n";
				return false;
			}
		}
		if( $ggLDAPDebug ) print "Successfully modified the user's groups<br />\n";
		
		return true;
	}
	
	/**
	 * Binds as $userdn with $password. This can be called with only the ldap
	 * connection resource for an anonymous bind.
	 * 
	 * @param resource $ldapconn
	 * @param string $userdn
	 * @param string $password
	 * @return bool
	 * @access private
	 */
	 private function bindAs( $ldapconn, $userdn, $password ) {
		//Let's see if the user can authenticate.
		if ( $userdn == null || $password == null ) {
			$bind = @ldap_bind( $ldapconn );
		} else {
			$bind = @ldap_bind( $ldapconn, $userdn, $password );
		}
		if ( !$bind ) return false;
		return true;
	}
	
	/**
	 * Returns a password that is created via the configured hash settings.
	 * 
	 * @param string $password
	 * @return string
	 * @access private
	 */
	private function getPasswordHash( $password ) {
		global $ggLDAPPasswordHash, $ggLDAPDebug;
		
		if( $ggLDAPDebug ) print "Entering getPasswordHash<br />\n";
		
		if ( isset( $ggLDAPPasswordHash ) ) {
			$hashtouse = $ggLDAPPasswordHash;
		} else {
			$hashtouse = "";
		}
		
		//Set the password hashing based upon admin preference
		switch ( $hashtouse ) {
			case "crypt":
				$pass = "{CRYPT}" . crypt($password);
				break;
			case "clear":
				$pass = $password;
				break;
			default:
				$pwd_sha = base64_encode( pack( "H*", sha1($password) ) );
				$pass = "{SHA}".$pwd_sha;
				break;
		}
		
		if( $ggLDAPDebug ) print "Password is $pass<br />\n";
		return $pass;
	}
	
	/**
	 * Returns a string which has the chars *, (, ), \ & NUL escaped to LDAP compliant
	 * syntax as per RFC 2254
	 * Thanks and credit to Iain Colledge for the research and function.
	 * 
	 * @param string $string
	 * @return string
	 * @access private
	 */
	private function getLdapEscapedString( $string ) {
		// Make the string LDAP compliant by escaping *, (, ) , \ & NUL
		return str_replace(
			array( "*", "(", ")", "\\", "\x00" ), //replace this
			array( "\\2a", "\\28", "\\29", "\\5c", "\\00" ), //with this
			$string //in this
		);
	}
	
	/**
	 * Fonction de dbuguage
	 * 
	 * @param string $string
	 * @return bool true
	 * @access private
	 */
	private function printDebug( $s ) {
		global $ggLDAPDebug;
		if( isset( $ggLDAPDebug ) && $ggLDAPDebug ) print $s."<br />\n";
		return true;
	}
}

